/*
 * Bundesliga is Copyright (c) 1997-2001 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

#define FROMMATCHDAY 0
#define FROMTABLE    1

Int16 matchDay, DBNr;
UInt32 Datum;
UInt32 fDatum[100];
MannStatus *MStatus = 0;
MemHandle MStatusHandle;
BlDatabasePreferencesType prefs;
BlApplicationPreferencesType appPrefs;
Boolean deviceCanBeam;

/*
 * array that holds pointers to the fields in the main form. m1 and m2 are
 * used to show the encounter, e1 and e2 hold the result. team1 and team2
 * store the teams's index.
 */
struct {
  FieldType *m1, *m2, *e1, *e2;
  Int8 team1, team2;
  UInt32 Datum;
} match[12];

static TablePtr leagueTable;
static TablePtr gotoTable;
static TablePtr fixtureTable;

// typedef char (*sortTableEntry)(MannStatus *a, MannStatus *b);

typedef Boolean (*PredicateF)(matchType*,Int8,Int8);

struct {
  Int16 m;
  Int16 first, last;
  PredicateF predicate;
} stats;

static Int16 TableRememberPosFirst = 0;
static Boolean showDay = true;

static Int8 getTeamAt(Coord, Coord, Int8);
static Boolean ComputeTable(void);

/**
 * UI helper function(s)
 */

void* Id2Ptr(UInt16 id)
{ 
  FormPtr frm = FrmGetActiveForm();
  return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,id));
}

void FrmMoveObject(Int16 Id,Int16 xd,Int16 yd) 
{
  UInt16 x,y;
  FormPtr frm = FrmGetActiveForm();
  UInt16 ObjectIndex = FrmGetObjectIndex(frm ,Id);
  FrmGetObjectPosition(frm, ObjectIndex, &x, &y);
  if (x+xd<160 && y+yd<160)
    FrmSetObjectPosition(frm, ObjectIndex, x+xd, y+yd);
}

ListPtr FrmGetListPtr(UInt16 frm, UInt16 id)
{
  FormPtr frmPtr = FrmGetFormPtr(frm);
  return FrmGetObjectPtr(frmPtr, FrmGetObjectIndex(frmPtr, id));
}

/*
 * __Attention__: don't forget to "MemPtrUnlock()" the returned
 * pointer after use!
 */
char* GetString(UInt16 id)
{
  MemHandle h = DmGetResource('tSTR', id);
  if (!h) return NULL;
  return MemHandleLock(h);
}

void DisplayErrorMessage(UInt16 id)
{
  char *err = GetString(id);
  FrmCustomAlert(ID_ErrorMessagesForm,err,NULL,NULL);
  MemPtrUnlock(err);
}

void DisplayInformationMessage(UInt16 id)
{
  char *err = GetString(id);
  FrmCustomAlert(ID_InformationMessagesForm,err,NULL,NULL);
  MemPtrUnlock(err);
}

void DisplayErrorMessageAndStop(UInt16 id)
{
  EventType e;
  DisplayErrorMessage(id);
  e.eType = appStopEvent;
  EvtAddEventToQueue(&e);
}

void RedrawTable(TablePtr tblP, UInt16 first,
		 Int8 rows, UInt16 up, UInt16 down)
{
  UInt16 row, last = first+9;
  ControlType *upP = Id2Ptr(up);
  ControlType *downP = Id2Ptr(down);
  char *label;

  if (last>rows-1) last=rows-1;
  for (row=0; row<TblGetNumberOfRows(tblP); row++)
    TblSetRowUsable(tblP, row, 
		    (row>=first && row<=last) ? true : false);

  label = (char*)CtlGetLabel(upP);
  label[0] = (first==0) ? 3 : 1;
  label = (char*)CtlGetLabel(downP);
  label[0] = (last==rows-1) ? 4 : 2;
  
  CtlDrawControl(upP);
  CtlDrawControl(downP);

  TblMarkTableInvalid(tblP);
  TblRedrawTable(tblP);
}

static char cmpTeamsByPtsGls(MannStatus *a, MannStatus *b)
{
  if (a->Pkte > b->Pkte) return 1;
  if (a->Pkte < b->Pkte) return -1;
  if (a->Tore + b->GTore > b->Tore + a->GTore) return 1;
  if (a->Tore + b->GTore < b->Tore + a->GTore) return -1;
  if (a->Tore > b->Tore) return 1;
  if (a->Tore < b->Tore) return -1;
  return 0;
}

static Boolean isHomeMatch(matchType *m, Int8 Mann, Int8 matchDay)
{
  return Mann == m->m1;
}

static Boolean isAwayMatch(matchType *m, Int8 Mann, Int8 matchDay)
{
  return Mann == m->m2;
}

static Boolean inBetween(matchType *m, Int8 Mann, Int8 matchDay)
{
  return (matchDay>=stats.first && matchDay<=stats.last);
}

/**
 * fixture form functions
 */
static void InitFormBl(void)
{
  Int16 i, fixup = 0, offset = 0, rows;

  for (i=0; i<12; i++) {
    match[i].m1 = Id2Ptr(ID_FrmMann+i);
    match[i].m2 = Id2Ptr(ID_FrmGegner+i);
    match[i].e1 = Id2Ptr(ID_FrmErg+i);
    match[i].e2 = Id2Ptr(ID_FrmGErg+i);
    FldSetUsable(match[i].m1, false);
    FldSetUsable(match[i].m2, false);
    FldSetUsable(match[i].e1, false);
    FldSetUsable(match[i].e2, false);
    CtlSetUsable(Id2Ptr(ID_FrmBlDateButton+i), false);
  }

  rows = prefs.NumTeams>>1;
  if (rows == 11) fixup = 1;
  if (rows == 10) fixup = 2;
  if (rows  < 10) { 
    fixup = 2;
    offset = 5;
  }

  if (fixup) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmMann+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmGegner+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmErg+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmGErg+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmBlDateButton+i,0,i*fixup+offset);
    }
    for (i=0; i<5; i++) {
      FrmMoveObject(ID_FrmButton+i,0,-fixup);
    }
  }

  ChooseDB();

  stats.predicate = inBetween;
  stats.first = 0;
  stats.last = prefs.NumMatchDays - 1;
}

void ButtonSetLabel(Int16 id, char *l)
{
  ControlType *ctlP = Id2Ptr(id);
  char *label;
  label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  StrCopy(label,l);
  CtlDrawControl(ctlP);
}

static void SetDateLabel(Int8 n)
{
  ControlType *ctlP = Id2Ptr(ID_FrmLabelMatchday);
  DateTimeType dateP;
  char *label;

  showDay = false;
  label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);

  TimSecondsToDateTime(match[n].Datum, &dateP);
  DateToAscii(dateP.month,dateP.day,dateP.year,
	      PrefGetPreference(prefDateFormat), label);

  // don't draw 0:00 time
  if (dateP.hour+dateP.minute>0) {
    StrCat(label," ");
    TimeToAscii(dateP.hour,dateP.minute,tfColon24h,
		label+StrLen(label));
  }
  CtlDrawControl(ctlP);
}

static void toggleMatchdayLabel(Boolean toggle)
{
  ControlType *ctlP = Id2Ptr(ID_FrmLabelMatchday);
  char *label;
  char md[3];

  label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  if (toggle)
    showDay = !showDay;

  if (showDay || !Datum) {
    char *p = GetString(ID_tSTR_Spieltag);
    StrIToA(md, matchDay);
    StrCopy(label,p);
    StrCat(label, " ");
    StrCat(label, md);
    MemPtrUnlock(p);
  } else {
    DateTimeType dateP;
    TimSecondsToDateTime(Datum, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
		PrefGetPreference(prefDateFormat), label);
  }
  CtlDrawControl(ctlP);
}

void SetResultHandle(FieldType* fptr, UInt8 n1, UInt8 n2)
{
  MemHandle h;
  if (!(h = FldGetTextHandle(fptr))) {
    h = MemHandleNew(4);
  }
  if (n1==255 && n2==255)
    StrCopy(MemHandleLock(h),"");
  else
    StrIToA(MemHandleLock(h), n1);
  MemHandleUnlock(h);
  FldSetTextHandle(fptr, h);
}

/*
 * Get the most used date from one match day.
 * hmm; way too long for such a simple thing :-/
 */
UInt32 GetMostUsedDate(void)
{
  struct count {
    UInt32 date;
    Int8 num;
  } date[12];

  UInt32 Datum;
  Int8 num = 0, i, j;
  Boolean newDate;

  for (i=0; i<12; i++) {
    if (match[i].Datum == 0)
      continue;
    newDate = true;
    for (j=0; j<num; j++) {
      if (date[j].date == match[i].Datum) {
	date[j].num++;
	newDate = false;
	break;
      }
    }
    if (newDate) {
      date[num].date = match[i].Datum;
      date[num].num = 1;
      num++;
    }
  }

  Datum = 0;
  i = 0;
  for (j=0; j<num; j++) {
    if (date[j].num > i) {
      Datum = date[j].date;
      i = date[j].num;
    }
  }
  return Datum;
}

void DrawMatches(Boolean redraw)
{
  Int8 i, numM, numH;
  char *p, *q;
  matchType *m;
  SelectedDBInfo info;
  FormPtr frm = FrmGetActiveForm();

  MStatus = MemHandleLock(MStatusHandle);
  p = q = MemHandleLock(DmQueryRecord(db,matchDay+1));

  numM = (Int8)*p++;
  numH = (Int8)*p++;

  for (i=0; i<12; i++) {

    match[i].team1 = match[i].team2 = match[i].Datum = 0;

    if (i<numM) {
      m = (matchType*)p;
      match[i].Datum = m->Datum;
    
      if ( m->m1==0 || m->m2==0 || 
	   m->m1>prefs.NumTeams || m->m2>prefs.NumTeams ) {
	DisplayErrorMessageAndStop(ID_ErrorMessages);
	MemPtrUnlock(q);
	MemHandleUnlock(MStatusHandle);
	prefs.Version = 0; // don't use this file again...
	return;
      }

      FldSetTextPtr(match[i].m1, MStatus[m->m1-1].team);
      FldSetTextPtr(match[i].m2, MStatus[m->m2-1].team);
      
      SetResultHandle(match[i].e1,m->e1,m->e2);
      SetResultHandle(match[i].e2,m->e2,m->e1);
      
      match[i].team1 = m->m1;
      match[i].team2 = m->m2;

      p += sizeof(matchType);

      FldSetUsable(match[i].m1, true);
      FldSetUsable(match[i].m2, true);
      FldSetUsable(match[i].e1, true);
      FldSetUsable(match[i].e2, true);
      CtlSetUsable(Id2Ptr(ID_FrmBlDateButton+i), true);
    } else {
      
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmMann+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmGegner+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmErg+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmGErg+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmBlDateButton+i));

    }
  }

  Datum = GetMostUsedDate();

  MemPtrUnlock(q);
  MemHandleUnlock(MStatusHandle);

  SelectedDB(DBNr, &info);
  FrmCopyLabel(frm, ID_FrmPopup, info.dbName);
  if (redraw) {
    FrmDrawForm(frm);
  }
  toggleMatchdayLabel(false);
}

/*
 * get the contents of the editable fields in the form and save them
 */
static void SaveMatchdayResults(void)
{
  Int16 i;
  char *e1, *e2;
  char *p, *q;
  Int8 numM;
  Boolean dirty = false;
  matchType *m;
  Int8 buf[2];

  if (!db || prefs.options & readonly)
    return;

  FrmSetFocus(FrmGetActiveForm(), -1);

  p = q = MemHandleLock(DmGetRecord(db,matchDay+1));
  numM = (Int8)*p;
  p += 2;

  for (i=0; i<numM; i++) {
    if (i>=12)
      break;

    m = (matchType*)p;

    e1 = FldGetTextPtr(match[i].e1);
    e2 = FldGetTextPtr(match[i].e2);

    buf[0] = buf[1] = 255;

    if (e1 && StrLen(e1) && e2 && StrLen(e2)) {
      buf[0] = StrAToI(e1);
      buf[1] = StrAToI(e2);
    }

    if (buf[0] != m->e1 || buf[1] != m->e2) {
      dirty = true;
      DmWrite(q,p-q+2,buf,2);
    }
    p += sizeof(matchType);
  }
  MemPtrUnlock(q);
  DmReleaseRecord(db,matchDay+1,dirty);
}

static FieldType* getResultFld(Coord x, Coord y)
{
  y = getY(y);
  if (x<70) 
    return match[y].e1;
  return match[y].e2;
}

Boolean SpieltagFormHandleEvent(EventPtr e)
{
  static Int16 first, last;
  Boolean handled;
  SelectedDBInfo info;
  Int16 m1, m2;
  
  handled = false;
  
  switch (e->eType) {

  case frmOpenEvent:
    InitFormBl();
    first = 0;
    last = prefs.NumMatchDays - 1;
    DrawMatches(true);
    handled = true; 
    break;

  case menuEvent:
    SaveMatchdayResults();
    switch(e->data.menu.itemID) {
    case ID_MenuItem:
      FrmPopupForm(ID_FrmAbout);
      handled = true; 
      break;

    case ID_MenuItem + 1:
      FrmPopupForm(ID_FrmTeams);
      handled = true; 
      break;

    case ID_MenuItem + 2:
      FrmPopupForm(ID_FrmEdit);
      handled = true; 
      break;

    case ID_MenuItem + 3:
      if (FrmAlert(ID_FrmDBLoeschen) == 0) { /* Lschen */
        TableRememberPosFirst = 0;
	if ( DeleteDatabase(DBNr) 
	     == (appErrorClass|noDatabaseInstalled) ) {
	  DisplayErrorMessageAndStop(ID_ErrorMessages+5);
	} else {
	  DBNr = 0;
	  SelectedDB(DBNr, &info);
	  OpenDatabase(0, 0);
	  matchDay = prefs.matchDay;
	  DrawMatches(false);
	  FrmGotoForm(ID_FrmBl);
	}
      }
      handled = true; 
      break;
    case ID_MenuItem + 4:
      FrmPopupForm(ID_FrmNewDB);
      handled = true; 
      break;
    case ID_MenuItem + 5:
      //FrmPopupForm(ID_FrmRenameLeague);
      FrmPopupForm(ID_FrmLeague);
      handled = true; 
      break;
    case ID_MenuItem + 6:
      FrmPopupForm(ID_FrmSettings);
      handled = true; 
      break;
    case ID_MenuItem + 7:
      FrmPopupForm(ID_FrmHandicap);
      handled = true; 
      break;
    case ID_MenuItem + 8:
      BeamDatabase(DBNr);
      handled = true; 
      break;
    }
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {
    case ID_FrmLabelMatchday:  /* Label upper right corner */
      toggleMatchdayLabel(true);
      handled = true; 
      break;

    case ID_FrmButton:  /* Table button */
      FrmGotoForm(ID_FrmTable);
      stats.first = first = 0;
      stats.last  = last = prefs.NumMatchDays - 1;
      handled = true; 
      break;

    case ID_FrmButton+4:
      {
	ListPtr pLst = Id2Ptr(ID_FrmPopup+2);
	LstSetSelection(pLst, 0);
	LstPopupList(pLst);
	switch (LstGetSelection(pLst)) {
	case 0: // table for day "matchDay"
	  first = 0;
	  last = matchDay-1;
	  handled = true; 
	  break;
	case 1: // second round
	  first = prefs.NumMatchDays>>1;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  break;
	case 2: // last 5 match days
	  first = matchDay - 5;
	  if (first<0) first = 0;
	  last = matchDay-1;
	  handled = true; 
	  break;
	case 3: // home matches
	  first = 0;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  stats.predicate = isHomeMatch;
	  break;
	case 4: // away matches
	  first = 0;
	  last = prefs.NumMatchDays - 1;
	  handled = true; 
	  stats.predicate = isAwayMatch;
	  break;
	}
	if (handled) {
	  FrmGotoForm(ID_FrmTable);
	  stats.first = first;
	  stats.last  = last;
	}
      }
      break;

    case ID_FrmButton+1: /* Prev-Button */
      if (matchDay>1) {
	SaveMatchdayResults();
        --matchDay;
        DrawMatches(true);
      }
      handled = true; 
      break;
    
    case ID_FrmButton+2: /* Next-Button */
      if (matchDay < prefs.NumMatchDays) {
        SaveMatchdayResults();
        ++matchDay;
        DrawMatches(true);
      }
      handled = true; 
      break;

    case ID_FrmButton+3: /* Goto-Button */
      FrmGotoForm(ID_FrmGoto);
      handled = true; 
      break;

    case ID_FrmPopup:
      {
	Int16 item;
	ListPtr pLst = Id2Ptr(ID_FrmPopup+1);
	LstSetSelection(pLst, DBNr);
	LstMakeItemVisible(pLst, DBNr);
	LstPopupList(pLst);
	if ( (item=LstGetSelection(pLst)) != DBNr && item!=-1) {
          TableRememberPosFirst = 0;
	  DBNr = item;
	  SaveMatchdayResults();
	  SelectedDB(DBNr, &info);
	  SaveStatus(DBNr);
	  if ( OpenDatabase(info.cardNo, info.dbID) ) {
	    DBNr = 0;
	    SelectedDB(DBNr, &info);
	    OpenDatabase(0,0);
	  }
	  matchDay = prefs.matchDay;
	  DrawMatches(false);
	  FrmGotoForm(ID_FrmBl);
	}
	handled = true;
      }
    default:
      if (e->data.ctlEnter.controlID >= ID_FrmBlDateButton &&
	  e->data.ctlEnter.controlID <= ID_FrmBlDateButton+11) {
	SetDateLabel(e->data.ctlEnter.controlID - ID_FrmBlDateButton);
      }
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (matchDay>1) {
        SaveMatchdayResults();
        --matchDay;
        DrawMatches(true);
      }
      handled = true;
      break;

    case pageDownChr:
      if (matchDay < prefs.NumMatchDays) {
        SaveMatchdayResults();
        ++matchDay;
        DrawMatches(true);
      }
      handled = true; 
    }
    break;

  case penDownEvent:
    m1 = getTeamAt(e->screenX, e->screenY, FROMMATCHDAY);
    if (m1>0) {
      handled = true;
      if (appPrefs.options & editMode) {
	Int16 e1 = 0, e2 = 0, n1, n2;
	FieldType *e1p, *e2p;
	if ( (e1p = getResultFld(e->screenX, e->screenY)) 
             // suggested by Rolf Muth <rolf.muth@ruhr-uni-bochum.de>
	     && FldGetTextLength(e1p)) 
	  e1 = ( StrAToI(FldGetTextPtr(e1p)) + 1 ) % 255;
	m2 = getTeamAt(135 - e->screenX, e->screenY, FROMMATCHDAY);
	if ( (e2p = getResultFld(135 - e->screenX, e->screenY)) )
	  e2 = StrAToI(FldGetTextPtr(e2p)) % 255;
	n1 = e1 + (e2<<8);
	n2 = e2 + (e1<<8);
	SetResultHandle(e1p,n1,n2);
	SetResultHandle(e2p,n2,n1);
	//FrmDrawForm(FrmGetActiveForm());
	FldDrawField(e1p);
	FldDrawField(e2p);
      } else {
	stats.m = m1;
	stats.first = first;
        stats.last = last;
	if (ComputeTable())
	  FrmPopupForm(ID_FrmStatistics);
      }
    }
    break;

  case frmCloseEvent:
    SaveMatchdayResults();
    break;

  default:
    break;
  }

  return handled;
}

/**
 * general Compute Table functions
 */

static void AddMStatus(Int8 n, Int8 n1, Int8 n2, Boolean home)
{
  Int8 pkte = (prefs.options & twoPoints) ? 2 : 3;
  if (n>0) n--;

  MStatus[n].Tore  += n1;
  MStatus[n].GTore += n2;
  MStatus[n].Spiele++;
  if (n1>n2) {
    MStatus[n].Pkte+=pkte;
    MStatus[n].Gew++;
  } else if (n1==n2) {
    MStatus[n].Pkte++;
    MStatus[n].Rem++;
  } else {
    MStatus[n].Ver++;
  }

  if (home) {
    MStatus[n].HTore  += n1;
    MStatus[n].HGTore += n2;
    MStatus[n].HSpiele++;
    if (n1>n2) {
      MStatus[n].HPkte+=pkte;
      MStatus[n].HGew++;
    } else if (n1==n2) {
      MStatus[n].HPkte++;
      MStatus[n].HRem++;
    } else {
      MStatus[n].HVer++;
    }
  }
}

static Boolean EvalResult(matchType *m, Int8 i)
{
  if ( m->m1==0 || m->m2==0 || 
       m->m1>prefs.NumTeams || m->m2>prefs.NumTeams ) {
    DisplayErrorMessageAndStop(ID_ErrorMessages);
    return false;
  }
  
  if (m->e1 != 0xff && m->e2 != 0xff) {
    if (stats.predicate(m,m->m1,i)) {
      AddMStatus(m->m1,m->e1,m->e2,true);
    }
    if (stats.predicate(m,m->m2,i)) {
      AddMStatus(m->m2,m->e2,m->e1,false);
    }
  }
  return true;
}

static Boolean EvalHandicap(handicapType *h, Int8 i)
{
  if (h->type == pointHandicap) {
    MStatus[h->m-1].Pkte += h->val;
  } else {
    MStatus[h->m-1].Tore += h->val;
  }
  return true;
}

Boolean LoopRecords(matchTypeCallback mCallback, 
		    handicapTypeCallback hCallback)
{
  Int8 i, j;
  Int8 numH, numM;
  char *p, *q;
  matchType *m;
  handicapType *h;

  for (i=0; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numM = (Int8)*p++;
    numH = (Int8)*p++;
    
    if (mCallback) {
      for (j=0; j<numM; j++) {
	m = (matchType*)p;
	
	if (!mCallback(m, i)) {
	  MemPtrUnlock(q);
	  prefs.Version = 0; // don't use this file again...
	  return false;
	}
	
	p += sizeof(matchType);
      }
    } else {
      p += numM*sizeof(matchType);
    }

    if (hCallback) {
      for (j=0; j<numH; j++) {
	h = (handicapType*)p;
	hCallback(h, i);
	p += sizeof(handicapType);
      }
    }

    MemPtrUnlock(q);
  }

  return true;
}

static Boolean ComputeTable()
{
  Int16 i,j;
  Boolean res;

  MStatus = MemHandleLock(MStatusHandle);
  for (i=0; i<prefs.NumTeams; i++) {
    MStatus[i].Tore = MStatus[i].GTore = MStatus[i].Pkte = MStatus[i].Spiele
      = MStatus[i].HSpiele = MStatus[i].Gew = MStatus[i].HGew
      = MStatus[i].Rem = MStatus[i].HRem = MStatus[i].Ver = MStatus[i].HVer
      = MStatus[i].HTore = MStatus[i].HGTore = MStatus[i].HPkte = 0;
    MStatus[i].Platz = 1;
  }
  if ( (res = LoopRecords(EvalResult,EvalHandicap)) ) {

    for (i=0; i<prefs.NumTeams-1; i++)
      for (j=i+1; j<prefs.NumTeams; j++) {
	char sort = cmpTeamsByPtsGls(&MStatus[i],&MStatus[j]);
	if (sort<0)
	  MStatus[i].Platz++;
	else if (sort>0)
	  MStatus[j].Platz++;
      }
  }
  MemHandleUnlock(MStatusHandle);

  return res;
}

/**
 * Form Compute Table Functions
 */

static void DrawTableItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n;
  UInt16 x;
  Char buf[25];

  n = TblGetItemInt(tableP, row, column);
  x = bounds->topLeft.x;

  StrIToA(buf,n);

  switch (column) {
  case 0:
    StrCat(buf,".");
    break;
  case 1:
    MStatus = MemHandleLock(MStatusHandle);
    StrCopy(buf, MStatus[n].team);
    MemHandleUnlock(MStatusHandle);
    if (n+1 == prefs.favTeam)
      WinSetUnderlineMode(solidUnderline);
    break;
  case 3:
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;
  }

  FntSetFont(0);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinSetUnderlineMode(noUnderline);
}  

/*
 * initialize table variables to default values;
 */
static void InitFormTable(void)
{
  Int16 i, j;
  leagueTable = Id2Ptr(ID_FrmTableTable);

  for (i=0; i<prefs.NumTeams; i++) {
    TblSetRowSelectable(leagueTable, i, false);
    for (j=0; j<6; j++) {
      TblSetItemStyle(leagueTable, i, j, customTableItem);
      TblSetColumnUsable(leagueTable, j, true);
      TblSetCustomDrawProcedure(leagueTable, j, DrawTableItem);
    }
  }
  for (i=prefs.NumTeams; i<TblGetNumberOfRows(leagueTable); i++) {
    TblSetRowUsable(leagueTable, i, false);
  }
}

/*
 * fills in the values for the table and redraws it.
 */
static void InitTable(void)
{
  Int16 Platz, n, row=0;
 
  MStatus = MemHandleLock(MStatusHandle);
  for (Platz=1; Platz<=prefs.NumTeams; Platz++) {
    for (n=0; n<prefs.NumTeams; n++) {
      if (MStatus[n].Platz==Platz) {
        TblSetItemInt(leagueTable, row, 0, Platz);
        TblSetItemInt(leagueTable, row, 1, n);
        TblSetItemInt(leagueTable, row, 2, MStatus[n].Spiele);
        TblSetItemInt(leagueTable, row, 3, MStatus[n].Tore);
        TblSetItemInt(leagueTable, row, 4, MStatus[n].GTore);
        TblSetItemInt(leagueTable, row, 5, MStatus[n].Pkte);
	TblSetRowUsable(leagueTable, row, false);
        row++;
      }
    }    
  }
  MemHandleUnlock(MStatusHandle);

  FrmDrawForm(FrmGetActiveForm());
}

Boolean TableFormHandleEvent(EventPtr e)
{
  Int8 m;
  Boolean handled;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    InitFormTable();
    if (ComputeTable()) {
      InitTable();
      RedrawTable(leagueTable, TableRememberPosFirst, prefs.NumTeams,
		  ID_FrmTableButton+1, ID_FrmTableButton+2);
    }
    handled  = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {
    case ID_FrmTableButton:
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;
    
    case ID_FrmTableButton+1:
      if (TableRememberPosFirst > 0) {
	TableRememberPosFirst -= 10;
	if (TableRememberPosFirst<0) 
	  TableRememberPosFirst = 0;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    
    case ID_FrmTableButton+2:
      if (TableRememberPosFirst+10 < prefs.NumTeams) {
	TableRememberPosFirst += 10;
	if (TableRememberPosFirst+10 > prefs.NumTeams)
	  TableRememberPosFirst = prefs.NumTeams - 10;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (TableRememberPosFirst > 0) {
	TableRememberPosFirst -= 10;
	if (TableRememberPosFirst<0) 
	  TableRememberPosFirst = 0;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (TableRememberPosFirst+10 < prefs.NumTeams) {
	TableRememberPosFirst += 10;
	if (TableRememberPosFirst+10 > prefs.NumTeams)
	  TableRememberPosFirst = prefs.NumTeams - 10;
	RedrawTable(leagueTable,
		    TableRememberPosFirst, 
		    prefs.NumTeams,
		    ID_FrmTableButton+1,
		    ID_FrmTableButton+2);
      }
      handled = true; 
      break;
    }
    break;

  case penDownEvent:
    MStatus = MemHandleLock(MStatusHandle);
    m = getTeamAt(e->screenX, e->screenY, FROMTABLE+TableRememberPosFirst);
    MemHandleUnlock(MStatusHandle);
    if (m>0) {
      handled = true;
      stats.m = m;
      FrmPopupForm(ID_FrmStatistics);
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * UI Goto Funktionen
 */

static void InitFormGoto(void)
{
  Int16 row, col;
  gotoTable = Id2Ptr(ID_FrmGoto);
  for (row=0; row<TblGetNumberOfRows(gotoTable); row++) {
    TblSetRowUsable(gotoTable, row, 8*row>=prefs.NumMatchDays?false:true);
    TblSetRowSelectable(gotoTable, row, 
			8*row>=prefs.NumMatchDays?false:true);
    for (col=0; col<8; col++) {
      TblSetColumnUsable(gotoTable, col, true);
      if (row*8+col < prefs.NumMatchDays) {
        TblSetItemStyle(gotoTable, row, col, numericTableItem);
        TblSetItemInt(gotoTable, row, col, row*8+col+1);
      } else
        TblSetItemStyle(gotoTable, row, col, customTableItem);
    }
  }
}

Boolean GotoFormHandleEvent(EventPtr e)
{
  UInt16 r,c;
  Boolean handled;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    InitFormGoto();
    FrmDrawForm(FrmGetActiveForm());
    TblDrawTable(gotoTable);
    handled = true; 
    break;

  case tblSelectEvent:
    r = e->data.tblSelect.row;
    c = e->data.tblSelect.column;
    if (r*8+c+1 <= prefs.NumMatchDays) {
      matchDay = r*8 + c + 1;
      FrmGotoForm(ID_FrmBl);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}

/**
 * About Form
 */
Boolean AboutFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmAboutButton) {
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  default:
    break;
  }

  return handled;
}


/**
 * ShowFixture Form
 */

static void DrawFixtureItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n;
  UInt16 x;
  Char buf[25], *p;
  DateTimeType dateP;

  n = TblGetItemInt(tableP, row, column);
  x = bounds->topLeft.x;

  FntSetFont(0);

  switch (column) {
  case 0:
    TimSecondsToDateTime(n<100?fDatum[n]:0, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
		PrefGetPreference(prefDateFormat), buf);
    StrCat(buf, " ");
    TimeToAscii(dateP.hour,dateP.minute,
		tfColon24h, buf+StrLen(buf));
    break;
  case 1:
    MStatus = MemHandleLock(MStatusHandle);
    if (n%256 == stats.m) {
      p = GetString(ID_tSTR_away);
      StrCopy(buf, p);
      StrCat(buf, MStatus[n/256-1].team);
    } else {
      p = GetString(ID_tSTR_home);
      StrCopy(buf, p);
      StrCat(buf, MStatus[n%256-1].team);
    }
    MemPtrUnlock(p);
    MemHandleUnlock(MStatusHandle);
    break;
  case 2:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n/256);
    }
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;
  case 3:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n%256);
    }
    break;
  }
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
}  

static Int16 InitFormShowFixture(void)
{
  Int16 col, row, i, j;
  char *p, *q;
  matchType *m;
  Int8 numM;
  UInt16 erg;

  fixtureTable = Id2Ptr(ID_FrmShowFixtureTable);

  for (row=0; row<TblGetNumberOfRows(fixtureTable); row++) {
    TblSetRowSelectable(fixtureTable, row, true);
    TblSetRowUsable(fixtureTable, row, false);
    for (col=0; col<4; col++) {
      TblSetItemStyle(fixtureTable, row, col, customTableItem);
      TblSetColumnUsable(fixtureTable, col, true);
      TblSetCustomDrawProcedure(fixtureTable, col, DrawFixtureItem);
    }
  }

  row = 0;
  for (i=0; i<prefs.NumMatchDays; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numM = (Int8)*p;
    p += 2;

    for (j=0; j<numM; j++) {
      m = (matchType*)p;
      if ( m->m1 == stats.m || m->m2 == stats.m ) {
	if ( m->m1 == stats.m ) {
	  erg = (m->e1<<8)+m->e2;
	} else {
	  erg = (m->e2<<8)+m->e1;
	}
	TblSetItemInt(fixtureTable, row, 0, row);
	if (row<100) fDatum[row] = m->Datum;
	TblSetItemInt(fixtureTable, row, 1, (m->m1<<8)+m->m2);
	TblSetItemInt(fixtureTable, row, 2, erg);
	TblSetItemInt(fixtureTable, row, 3, erg);
	row++;
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }

  FrmDrawForm(FrmGetActiveForm());
  TblDrawTable(fixtureTable);
  return row;
}

Boolean ShowFixtureFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Int8 from, rows;
  UInt16 row, column;
  char* title;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    title = (char*)FrmGetTitle(FrmGetActiveForm());
    MStatus = MemHandleLock(MStatusHandle);
    while (*title++ != ' ');
    MemMove(title, MStatus[stats.m-1].team, 
	    StrLen(MStatus[stats.m-1].team));
    MemHandleUnlock(MStatusHandle);
    rows = InitFormShowFixture();
    from = 0;

    RedrawTable(fixtureTable, from,rows,
		ID_FrmShowFixtureButton+1,
		ID_FrmShowFixtureButton+2);

    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmShowFixtureButton:
      FrmGotoForm(ID_FrmStatistics);
      handled = true; 
      break;
    
    case ID_FrmShowFixtureButton+1:
      if (from > 0) {
	from -= 10;
	if (from < 0) from = 0;
      }
      RedrawTable(fixtureTable, from, rows,
		  ID_FrmShowFixtureButton+1,
		  ID_FrmShowFixtureButton+2);
      handled = true; 
      break;
    
    case ID_FrmShowFixtureButton+2:
      if (from+10 < rows) {
	from += 10;
	if (from+10>rows) from = rows - 10;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (from>0) {
	from -= 10;
	if (from < 0) from = 0;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (from+10 < rows) {
	from += 10;
	if (from+10>rows) from = rows - 10;
	RedrawTable(fixtureTable, from, rows,
		    ID_FrmShowFixtureButton+1,
		    ID_FrmShowFixtureButton+2);
      }
      handled = true; 
      break;
    }
    break;

  case tblSelectEvent:
    TblGetSelection(fixtureTable, &row, &column);
    if (column == 1) {
      UInt16 val = TblGetItemInt(fixtureTable, row, 1);
      if (val%256 == stats.m)
        stats.m = val/256;
      else
        stats.m = val%256;
      FrmGotoForm(ID_FrmShowFixture);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}


/**
  * Statistics Form Handling Routines
  */
Int16 getY(Int16 y)
{
  Int16 div = 10 , offset = 0, rows = prefs.NumTeams>>1;

  if (rows==11)
    div = 11;
  if (rows==10)
    div = 12;
  if (rows<10) {
    div = 12;
    offset = 5;
  }

  y = (y-24-offset) / div;
  return y;
}

static Int8 getTeamAt(Coord x, Coord y, Int8 From)
{
  Int8 i, n, row;
  switch(From) {
  case FROMMATCHDAY:
    if (y<24 || y>146 || (x>62 && x<72) || x>135)
      return 0;
    y = getY(y);

    if (x<70) 
      return match[y].team1;
    return match[y].team2;
    break;

  default:
    if (y<30 || y>139)
      return 0;
    y = (y-30)/11 + From;

    row = 0;
    for (i=1; i<=prefs.NumTeams; i++) {
      for (n=0; n<prefs.NumTeams; n++) {
	if (MStatus[n].Platz==i) {
	  row++;
	  if (row == y)
	    return n+1;
	}
      }    
    }

    break;
  }
  return 0;
}

static void DrawIntAt(Int16 v, Coord x, Coord y)
{
  Char buf[35];
  StrIToA(buf,v);
  WinDrawChars(buf,StrLen(buf),x,y);
}

static void GetGamesString(Char* buf, Int16 n)
{
  Int8 i, j, numM;
  char *p, *q, *r;
  matchType *m;


  StrCopy(buf,"");
  for (i=stats.first; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numM = (Int8)*p;
    p+=2;

    for (j=0; j<numM; j++) {
      m = (matchType*)p;
      if (m->m1 == n || m->m2 == n) {

	if (stats.predicate(m,n,i)) {
	  if (m->e1 != 0xff && m->e2 != 0xff) {
	    if (m->m1 == n) { // home match
	      if (m->e1>m->e2) {
		r = GetString(ID_tSTR_H_WON);
	      } else if (m->e1 == m->e2) {
		r = GetString(ID_tSTR_H_DRAW);
	      } else {
		r = GetString(ID_tSTR_H_LOST);
	      }
	    } else { // away match
	      if (m->e2 > m->e1) {
		r = GetString(ID_tSTR_A_WON);
	      } else if (m->e1 == m->e2) {
		r = GetString(ID_tSTR_A_DRAW);
	      } else {
		r = GetString(ID_tSTR_A_LOST);
	      }
	    }
	    StrCat(buf,r);
	    MemPtrUnlock(r);
	  } else {
	    StrCat(buf, "-");
	  }
	}
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }
}

Boolean StatisticsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  Char *title;
  Char buf[91];
  Char Platz[3];
  Int16 i, k;
  
  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    MStatus = MemHandleLock(MStatusHandle);

    FntSetFont(0);
    // modify title, set team name
    title = (char*)FrmGetTitle(FrmGetActiveForm());
    while ( *title++ != ' ' );
    MemMove(title, MStatus[stats.m-1].team, 
	    StrLen(MStatus[stats.m-1].team));
    StrIToA(Platz, MStatus[stats.m-1].Platz);
    FldSetTextPtr(Id2Ptr(ID_FrmStatisticsFld), Platz);
    FrmDrawForm(FrmGetActiveForm());

    DrawIntAt(MStatus[stats.m-1].Spiele,5,37);
    DrawIntAt(MStatus[stats.m-1].Gew,20,37);
    DrawIntAt(MStatus[stats.m-1].Rem,35,37);
    DrawIntAt(MStatus[stats.m-1].Ver,50,37);
    StrIToA(buf,MStatus[stats.m-1].Tore);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),MStatus[stats.m-1].GTore);
    WinDrawChars(buf,StrLen(buf),65,37);
    DrawIntAt(MStatus[stats.m-1].Tore-MStatus[stats.m-1].GTore,95,37);
    DrawIntAt(MStatus[stats.m-1].Pkte,115,37);
    
    DrawIntAt(MStatus[stats.m-1].HSpiele,5,71);
    DrawIntAt(MStatus[stats.m-1].HGew,20,71);
    DrawIntAt(MStatus[stats.m-1].HRem,35,71);
    DrawIntAt(MStatus[stats.m-1].HVer,50,71);
    StrIToA(buf,MStatus[stats.m-1].HTore);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),MStatus[stats.m-1].HGTore);
    WinDrawChars(buf,StrLen(buf),65,71);
    DrawIntAt(MStatus[stats.m-1].HTore-MStatus[stats.m-1].HGTore,95,71);
    DrawIntAt(MStatus[stats.m-1].HPkte,115,71);

    DrawIntAt(MStatus[stats.m-1].Spiele-MStatus[stats.m-1].HSpiele,5,105);
    DrawIntAt(MStatus[stats.m-1].Gew-MStatus[stats.m-1].HGew,20,105);
    DrawIntAt(MStatus[stats.m-1].Rem-MStatus[stats.m-1].HRem,35,105);
    DrawIntAt(MStatus[stats.m-1].Ver-MStatus[stats.m-1].HVer,50,105);
    StrIToA(buf,MStatus[stats.m-1].Tore-MStatus[stats.m-1].HTore);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),MStatus[stats.m-1].GTore-MStatus[stats.m-1].HGTore);
    WinDrawChars(buf,StrLen(buf),65,105);
    DrawIntAt(MStatus[stats.m-1].Tore-MStatus[stats.m-1].GTore
	      -MStatus[stats.m-1].HTore+MStatus[stats.m-1].HGTore,95,105);
    DrawIntAt(MStatus[stats.m-1].Pkte-MStatus[stats.m-1].HPkte,115,105);

    GetGamesString(buf,stats.m);
    k = prefs.NumMatchDays>>1;
    WinDrawChars(buf,(i=StrLen(buf))>k?k:i,5,117);
    if (i>k)
      WinDrawChars(buf+k,StrLen(buf+k),5,128);
    
    handled = true; 
    MemHandleUnlock(MStatusHandle);
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmStatisticsButton:
      FrmReturnToForm(0);
      handled = true; 
      break;

    case ID_FrmStatisticsButton+1:
      FrmGotoForm(ID_FrmShowFixture);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

