#ifndef _BL_H_
#define _BL_H_

/*
 * definitions from main.c
 */
extern UInt32 romversion;
#define FeatureSet20 (romversion >= 0x02003000)
#define FeatureSet30 (romversion >= 0x03003000)
#define FeatureSet31 (romversion >= 0x03103000)

/*
 * We put some functions into different code sections; we do this,
 * because linking with debugging enabled doesn't work anymore. The
 * produced _code_ extends 32KB then. Gdb has slight problems with 
 * debugging code in such a section, but fortunately it exists a 
 * patch, so this is not a problem anymore.
 */

#define EDITFUNC __attribute__ ((__section__("editfns")))
#define DBFUNC   __attribute__ ((__section__("dbfns")))

/*
 * definitions from bl.c
 */

typedef struct {
  UInt16 Version;
  Int16 matchDay;
  Int16 NumTeams;
  Int16 NumMatchDays;
  Int8 options;
# define twoPoints  1
# define readonly   2
  Int8 favTeam;
} BlDatabasePreferencesType;

typedef struct {
  Int16 Version;
  LocalID dbID;
  UInt16 cardNo;
  Int8 options;
# define editMode   1
# define dontSaveEM 2
  UInt32 dbHash[20];
} BlApplicationPreferencesType;

extern Int16 DBNr;
extern BlApplicationPreferencesType appPrefs;
extern BlDatabasePreferencesType prefs;

typedef struct {
  UInt16 Tore, GTore, Pkte,  HTore, HGTore, HPkte; 
  Int8 Spiele, Platz, HSpiele, Gew, HGew, Rem, HRem, Ver, HVer;
  char *team;
} MannStatus;

typedef struct {
  UInt8 m1, m2, e1, e2;
  UInt32 Datum;
} matchType;

typedef struct {
  Int8 type, m;
  Char val;
# define pointHandicap 0
} handicapType;

typedef Boolean (*matchTypeCallback)(matchType*,Int8);
typedef Boolean (*handicapTypeCallback)(handicapType*,Int8);

extern MannStatus *MStatus;
extern MemHandle MStatusHandle;

extern Int16 matchDay;

Boolean SpieltagFormHandleEvent(EventPtr);
Boolean TableFormHandleEvent(EventPtr);
Boolean GotoFormHandleEvent(EventPtr);
Boolean AboutFormHandleEvent(EventPtr);
Boolean StatisticsFormHandleEvent(EventPtr);
Boolean ShowFixtureFormHandleEvent(EventPtr);

void* Id2Ptr(UInt16);
void FrmMoveObject(Int16, Int16, Int16);
void ButtonSetLabel(Int16 id, char *l);
void DrawMatches(Boolean);
ListPtr FrmGetListPtr(UInt16 frm, UInt16 id);
void DisplayErrorMessage(UInt16 stringId);
void DisplayInformationMessage(UInt16 stringId);
char* GetString(UInt16 id);
Boolean LoopRecords(matchTypeCallback,handicapTypeCallback);
void RedrawTable(TablePtr tblP, UInt16 first,
		 Int8 rows, UInt16 up, UInt16 down);
Int16 getY(Int16 y);

/*
 * definitions from dbfuncs.c
 */
typedef struct {
  LocalID dbID;
  UInt16 cardNo;
  Char dbName[32];
  UInt16 dbAttr;
} SelectedDBInfo;

extern char* Items[];
extern Int16 nrOfItems;
extern DmOpenRef db;

#define tooOld              1
#define noDatabaseInstalled 2

DmOpenRef CreateDatabase(char*) DBFUNC;
Err ReadDatabase(DmOpenRef) DBFUNC;
Err OpenDatabase(UInt16, LocalID) DBFUNC;
void SavePreferences(void) DBFUNC;
void SaveStatus(Int16) DBFUNC;
Int16 FindInstalledDatabases(void) DBFUNC;
void ChooseDB(void) DBFUNC;
void SelectedDB(Int16, SelectedDBInfo*) DBFUNC;
void InsertNewRecord(DmOpenRef, UInt16*, UInt16) DBFUNC;
void InitNewDatabase(DmOpenRef,BlDatabasePreferencesType*) DBFUNC;
Err DeleteDatabase(Int16) DBFUNC;
Boolean RenameDatabase(Int16 i, char* name) DBFUNC;
void swapDbInfo(Int16, Int16) DBFUNC;
void BeamDatabase(Int16) DBFUNC;

/*
 * definitions from edit.c
 */
Boolean EditFormHandleEvent(EventPtr) EDITFUNC;
Boolean TeamsFormHandleEvent(EventPtr) EDITFUNC;
Boolean EnterMannFormHandleEvent(EventPtr) EDITFUNC;
Boolean NewDBFormHandleEvent(EventPtr) EDITFUNC;
Boolean RenameLeagueFormHandleEvent(EventPtr) EDITFUNC;
Boolean SettingsFormHandleEvent(EventPtr) EDITFUNC;
Boolean HandicapFormHandleEvent(EventPtr e) EDITFUNC;
Boolean HandicapEditFormHandleEvent(EventPtr e) EDITFUNC;
Boolean LeagueFormHandleEvent(EventPtr e) EDITFUNC;

#endif
