/*
 * Bundesliga is Copyright (c) 1997-2001 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

struct {
  FieldType *m1, *m2;
  UInt32 date;
} editMatch[12];

static RectangleType eRect;
static PointType ePt;
static char* TeamsList[30];
static DateTimeType date;
static Int16 editMatchDay;
static TablePtr handicapTable;
static Int16 handicapTableRows;
static FieldType *lastField;

static void SetDateMatchdayLabel(void) EDITFUNC;
static void SetDateMatchday(Int8) EDITFUNC;
static void InitFormEdit(void) EDITFUNC;
static Boolean DrawFormEdit(void) EDITFUNC;
static void exchange(FieldType* m1, FieldType* m2) EDITFUNC;
static FieldType* getTeamUnder(FieldType* f) EDITFUNC;
static FieldType* teamUnder(Coord x, Coord y) EDITFUNC;
static void dragTeam(Coord x, Coord y) EDITFUNC;
static Int16 MNrvString(char* s) EDITFUNC;
static void UpdateSpielplan(Boolean) EDITFUNC;
static void InitFormTeams(void) EDITFUNC;
static Int16 TeamsListGetSelection(void) EDITFUNC;
static void RedrawList(UInt16 ID_Form, UInt16 ID_List) EDITFUNC;
static void SetField(UInt16 ID, Int16 size, char* val) EDITFUNC;
static void DrawFormEnterMann(void) EDITFUNC;
static void SetEnteredMann(Int8) EDITFUNC;
static void InitFrmNewDB(void) EDITFUNC;
static void ReadFormNewDBValues(void) EDITFUNC;
static void InitFrmRenameLeague(void) EDITFUNC;
static void RenameDatabaseFile(void) EDITFUNC;
static void updateDbNr(Int16 i, Int16 j) EDITFUNC;
static void DrawHandicapItem(void*, Int16, Int16, RectangleType*) EDITFUNC;
static void InitFormHandicap(void) EDITFUNC;
static Boolean GetHandicap(handicapType *h, Int8 i) EDITFUNC;
static Int8 numberOfHandicapEntries(Int8 spTag) EDITFUNC;
static void GetNextHandicap(handicapType *h, Int8 spTag) EDITFUNC;
static void SaveHandicaps(void) EDITFUNC;
static void SetTeamsList(void) EDITFUNC;

/**
 * Edit Form
 * purpose: edit the fixture by drag and drop
 */
static void SetDateMatchdayLabel()
{
  static Char buf[3];
  ControlPtr ctlP = Id2Ptr(ID_FrmEditDatum);
  char *label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  DateToAscii(date.month, date.day, date.year, 
  	      PrefGetPreference(prefDateFormat), label);

  ctlP = Id2Ptr(ID_FrmEditDatum+1);
  label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  TimeToAscii(date.hour, date.minute,
  	      PrefGetPreference(prefTimeFormat), label);

  StrIToA(buf, editMatchDay);
  FldSetTextPtr(Id2Ptr(ID_FrmEditFld), buf);
}

static void SetDateMatchday(Int8 fld)
{
  UInt8 i;
  if (fld != -1) {
    editMatch[fld].date = TimDateTimeToSeconds(&date);
  } else {
    for (i=0; i<12; i++) {
      editMatch[i].date = TimDateTimeToSeconds(&date);
    }
  }
}

static void InitFormEdit()
{
  Int16 i, fixup=0, rows, offset=0;

  for (i=0; i<12; i++) {
    editMatch[i].m1 = Id2Ptr(ID_FrmEditMann+i);
    editMatch[i].m2 = Id2Ptr(ID_FrmEditMann+i+20);
  }

  rows = (prefs.NumTeams+1)>>1;
  if (rows == 11) fixup = 1;
  if (rows == 10) fixup = 2;
  if (rows  < 10) { 
    fixup = 2;
    offset = 5;
  }

  if (fixup) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmEditMann+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmEditMann+20+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmEditMann+40+i,0,i*fixup+offset);
    }
    for (i=0; i<5; i++) {
      FrmMoveObject(ID_FrmEditButton+i,0,-fixup);
    }
  }
}

static Boolean DrawFormEdit()
{
  UInt32 mask = 0UL, Datum = 0;
  Int8 i, anzS;
  char *p, *q;
  matchType *m;
  Boolean hasResults = false;

  MStatus = MemHandleLock(MStatusHandle);
  p = q = MemHandleLock(DmQueryRecord(db,editMatchDay+1));
  anzS = (Int8)*p;
  p+=2;

  for (i=0; i<12; i++) {
    if (i<anzS) {
      m = (matchType*)p;
      if (!Datum)
	Datum = m->Datum;
    
      FldSetTextPtr(editMatch[i].m1, MStatus[m->m1-1].team);
      FldSetUsable(editMatch[i].m1, true);
      FldSetTextPtr(editMatch[i].m2, MStatus[m->m2-1].team);
      FldSetUsable(editMatch[i].m2, true);
      editMatch[i].date = m->Datum;
      CtlSetUsable(Id2Ptr(ID_FrmEditMann+40+i), true);

      /* remember which teams play this round */
      mask |= ( 1UL<<(m->m1-1) ) | ( 1UL<<(m->m2-1) );
      
      hasResults = hasResults || (m->e1 != 0xff && m->e2 != 0xff);
      
      p += sizeof(matchType);
    } else {
      FormPtr frm = FrmGetActiveForm();
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditMann+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditMann+20+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditMann+40+i));
    }

  }

  lastField = editMatch[anzS-1].m2;

  if (prefs.NumTeams % 2 == 1) {
    for (i=0; i<prefs.NumTeams; i++) {
      /* set team that doesn't play this round */
      if ( (mask & (1UL<<i)) == 0 ) { 
	FldSetTextPtr(editMatch[prefs.NumTeams>>1].m2,MStatus[i].team);
	FldSetUsable(editMatch[prefs.NumTeams>>1].m2, true);
	lastField = editMatch[prefs.NumTeams>>1].m2;
	break;
      }
    }
  }
  MemPtrUnlock(q);
  MemHandleUnlock(MStatusHandle);

  if (Datum == 0) {
    TimSecondsToDateTime(TimGetSeconds(), &date);
  } else {
    TimSecondsToDateTime(Datum, &date);
  }
  SetDateMatchdayLabel();

  return hasResults;
}

/*
 * swap contents of the two fields
 */
static void exchange(FieldType* m1, FieldType* m2)
{
  char *s1, *s2;
  s1 = FldGetTextPtr(m1);
  s2 = FldGetTextPtr(m2);
  FldSetTextPtr(m1, s2);
  FldSetTextPtr(m2, s1);
  FldDrawField(m1);
  FldDrawField(m2);
}

static FieldType* getTeamUnder(FieldType* f)
{
  FieldAttrType attr;
  FldGetAttributes(f, &attr);
  if (!attr.usable) /* is field used */
    return NULL;
  FldGetBounds(f, &eRect);
  return f;
}

static FieldType* teamUnder(Coord x, Coord y)
{
  if (y<24 || y>146 || (x>70 && x<88) || x>153 || x<5)
    return NULL;
  y = getY(y);
  if (y<0 || y>11)
    return NULL;

  if (x<70) { /* left half  */
    return getTeamUnder(editMatch[y].m1);
  }
  /* right half */
  return getTeamUnder(editMatch[y].m2);
}

static void dragTeam(Coord x, Coord y)
{
  WinInvertRectangle(&eRect, 2);
  RctOffsetRectangle(&eRect, x-ePt.x, y-ePt.y);
  WinInvertRectangle(&eRect, 2);
  ePt.x = x;
  ePt.y = y;
}

/*
 * return number of team in team table of argument "s"
 */
static Int16 MNrvString(char* s)
{
  Int8 i,ret;

  /*
   * We've only mangled pointers, so let's compare the pointers
   */
  ret = 0;
  MStatus = MemHandleLock(MStatusHandle);
  for (i=0; i<prefs.NumTeams; i++) {
    if (MStatus[i].team == s) {
      ret = i+1;
      break;
    }
  }
  MemHandleUnlock(MStatusHandle);
  return ret;
}

/*
 * correct settings in fixture according to drag & drop actions before;
 * if necessary reset results - the user was warned about this!
 */
static void UpdateSpielplan(Boolean add)
{
  matchType *m,mn;
  Int8 numM, numH, i, m1, m2;
  char *p, *q;
  UInt16 size;

  if (prefs.options & readonly)
    return;

  p = MemHandleLock(DmQueryRecord(db,editMatchDay+1));
  numM = (Int8)*p;
  numH = (Int8)*(p+1);
  MemPtrUnlock(p);

  if (numM>=9)
    add = false;

  size = 2 + numM*sizeof(matchType) + numH*sizeof(handicapType);

  if (add)
    size += sizeof(matchType);

  p = q = MemHandleLock(DmResizeRecord(db, editMatchDay+1, size));
  p += 2;

  for (i=0; i<numM; i++) {
    m = (matchType*)p;

    m1 = MNrvString(FldGetTextPtr(editMatch[i].m1));
    m2 = MNrvString(FldGetTextPtr(editMatch[i].m2));
    if (m1 && m2) {
      mn.m1 = m1; 
      mn.m2 = m2; 
      mn.Datum = editMatch[i].date;
      mn.e1 = mn.e2 = 0xff;
      DmWrite(q,p-q,&mn,sizeof(matchType));
    }
    p += sizeof(matchType);
  }

  if (add) {
    numM++;
    DmWrite(q,0,&numM,1);
    DmWrite(q,p-q+sizeof(matchType),p,numH*sizeof(handicapType));
    DmWrite(q,p-q,&mn,sizeof(matchType));
  }

  MemPtrUnlock(q);
  DmReleaseRecord(db,editMatchDay+1,true);
}

Boolean EditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static FieldType* lastTeam;
  FieldType* newTeam;
  static ListType *pLst;
  static Boolean hasChanged = false;
  static Int8 dateButton;
  UInt16 index;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFormEdit();
    handled = true;
    editMatchDay = matchDay;
    dateButton = -1;
    if (DrawFormEdit()) {
      if (FrmAlert(ID_FrmHasResults) == 1) {
        /* don't delete results, but return to form immediately */
        FrmReturnToForm(0);
        break;
      }
    }
    FrmDrawForm(FrmGetActiveForm());
    SetTeamsList();
    pLst = Id2Ptr(ID_FrmEditList);
    LstSetHeight(pLst, prefs.NumTeams>10?10:prefs.NumTeams);
    LstSetListChoices(pLst, TeamsList, prefs.NumTeams);
    break;

  case menuEvent:
    switch(e->data.menu.itemID) {
    case ID_EditMenuItem:    /* Add Match */
      UpdateSpielplan(true); /* write record and add a match */
      matchDay = editMatchDay;
      FrmGotoForm(ID_FrmEdit);
      handled = true;
      break;

    case ID_EditMenuItem+1: /* Add Match Day */
      if (prefs.NumMatchDays<90) {
	index = editMatchDay+3;
	prefs.NumMatchDays++;
	InsertNewRecord(db, &index, prefs.NumTeams>>1);
      }
      handled = true;
      break;

    case ID_EditMenuItem+2: /* Add Team */
      if (prefs.NumTeams < 24) {
	FormPtr savedForm = FrmGetActiveForm(),
	  frm = FrmInitForm(ID_FrmEnterMann);
	FrmSetActiveForm(frm);
	FrmDrawForm(frm);
	if (FrmDoDialog(frm) == ID_FrmEnterMannButton) {
	  char *team = FldGetTextPtr(Id2Ptr(ID_FrmEnterMann));
	  if (team && StrLen(team)) {
	    prefs.NumTeams++;
	    if (CtlGetValue(Id2Ptr(ID_FrmEnterMannButton+2))) {
	      prefs.favTeam = prefs.NumTeams;
	    }
	    SavePreferences();
	    MemHandleResize(MStatusHandle, prefs.NumTeams*sizeof(MannStatus));
	    SetEnteredMann(prefs.NumTeams-1);
	    matchDay = editMatchDay;
	    FrmGotoForm(ID_FrmEdit);
	  }
	}
	FrmDeleteForm(frm);
	FrmSetActiveForm(savedForm);
      }
      handled = true;
      break;
    }
    break;

  case penDownEvent:
    /* are we over a field with a team? */
    if ((lastTeam = teamUnder(e->screenX, e->screenY))) {
      WinInvertRectangle(&eRect, 2);
      ePt.x = e->screenX;
      ePt.y = e->screenY;
      handled = true;
    }
    break;

  case penMoveEvent:
    /* move rectangle */
    if (lastTeam) {
      dragTeam(e->screenX, e->screenY);
      handled = true;
    }
    break;

  case penUpEvent:
    /* swap teams */
    if (lastTeam) {
      WinInvertRectangle(&eRect, 2);
      if ((newTeam = teamUnder(e->screenX, e->screenY))) {
	exchange(lastTeam,newTeam);
        hasChanged = true;
	handled = true;
      }
    }
    break;

  case ctlSelectEvent:
    /* "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton) {
      if (hasChanged) {
        UpdateSpielplan(false);
      }
      FrmReturnToForm(0);
      if (hasChanged) 
	DrawMatches(true);
      handled = true; 
      break;
    }

    /* "Cancel" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+1) {
      FrmReturnToForm(0);
      handled = true; 
      break;
    }

    /* "Left" / "Right" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+2 ||
        e->data.ctlSelect.controlID == ID_FrmEditButton+3) {
      handled = true;
      if ( (editMatchDay > 1 
	    && e->data.ctlSelect.controlID == ID_FrmEditButton+2) 
         || (editMatchDay < prefs.NumMatchDays 
	     && e->data.ctlSelect.controlID == ID_FrmEditButton+3) ){
        if (hasChanged) {
          UpdateSpielplan(false);
          hasChanged = false;
        }
        editMatchDay += 
	  (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?-1:1;
        if (DrawFormEdit()) {
          if (FrmAlert(ID_FrmHasResults) == 1) {
            editMatchDay += 
	      (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?1:-1;
	    DrawFormEdit();
            break;
          }
        }
	if (dateButton != -1) {
	  ButtonSetLabel(ID_FrmEditMannDateButton+dateButton, "-");
	  dateButton = -1;
	}
        FrmDrawForm(FrmGetActiveForm());
      }
      break;
    }

    /* "Date" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDatum) {
      char *r = GetString(ID_tSTR_ChooseDate);
      if ( SelectDay(selectDayByDay, 
		     &date.month, &date.day, &date.year, r) ) {
	SetDateMatchdayLabel();
	SetDateMatchday(dateButton);
        FrmDrawForm(FrmGetActiveForm());
        hasChanged = true;
      }
      MemPtrUnlock(r);
      handled = true; 
      break;
    }

    /* "Time" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDatum+1) {
      char *r = GetString(ID_tSTR_ChooseTime);
      if (FeatureSet31) {
	if ( SelectOneTime(&date.hour, &date.minute, r) ) {
	  SetDateMatchdayLabel();
	  SetDateMatchday(dateButton);
	}
      } else {
	/* grr; why isn't SelectOneTime() exported before 3.1? 
	 * The result looks really really ugly... */
	TimeType st, et;
	st.hours = date.hour;
	st.minutes = date.minute;
	if ( SelectTimeV33(&st, &et, false, r, 11) ) {
	  date.hour = st.hours;
	  date.minute = st.minutes;
	  SetDateMatchdayLabel();
	  SetDateMatchday(dateButton);
	}
      }
      MemPtrUnlock(r);
      FrmDrawForm(FrmGetActiveForm());
      hasChanged = true;
      handled = true; 
      break;
    }

    /* "Team" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+4) {
      Int8 i;
      index=MNrvString(FldGetTextPtr(editMatch[((prefs.NumTeams+1)>>1)-1].m2));
      LstSetSelection(pLst, index-1);
      LstPopupList(pLst);
      i = LstGetSelection(pLst);
      MStatus = MemHandleLock(MStatusHandle);
      FldSetTextPtr(lastField, MStatus[i].team);
      MemHandleUnlock(MStatusHandle);
      FldDrawField(lastField);
      hasChanged = true;
      handled = true;
      break;
    }

    /* One of the "Date" buttons. */
    if (e->data.ctlSelect.controlID >= ID_FrmEditMannDateButton &&
	e->data.ctlSelect.controlID <= ID_FrmEditMannDateButton+11) {
      if (dateButton != -1) {
	ButtonSetLabel(ID_FrmEditMannDateButton+dateButton, "-");
      }
      dateButton = e->data.ctlSelect.controlID - ID_FrmEditMannDateButton;
      ButtonSetLabel(ID_FrmEditMannDateButton+dateButton, "=");
      TimSecondsToDateTime(editMatch[dateButton].date, &date);
      SetDateMatchdayLabel();
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;
    }

    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling editing and entering teams list
 */
static void SetTeamsList(void)
{
  Int16 i;
  MemHandleLock(MStatusHandle);
  for (i=0; i<prefs.NumTeams; i++) {
    TeamsList[i] = MStatus[i].team;
  }
  MemHandleUnlock(MStatusHandle);
}

static void InitFormTeams()
{
  SelectedDBInfo info;
  ListPtr p;

  SelectedDB(DBNr, &info);
  FrmCopyLabel(FrmGetActiveForm(), ID_FrmTeamsName, info.dbName);

  SetTeamsList();

  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  LstSetListChoices(p, TeamsList, prefs.NumTeams);
  LstSetHeight(p, 10);
}

static Int16 TeamsListGetSelection()
{
  ListPtr p;
  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  return LstGetSelection(p);
}

static void RedrawList(UInt16 ID_Form, UInt16 ID_List)
{
  ListPtr p;
  p = FrmGetListPtr(ID_Form, ID_List);
  LstEraseList(p);
  LstDrawList(p);
}

Boolean TeamsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFormTeams();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Button "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton) {
      FrmReturnToForm(0);
      DrawMatches(true);
      handled = true; 
      break;
    }

    /* Button "Edit" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton+1) {
      FrmPopupForm(ID_FrmEnterMann);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/*
 * helper function to acquire a handle if necessary and set contents
 * of a field.
 */
static void SetField(UInt16 ID, Int16 size, char* val)
{
  FieldType* fld;
  MemHandle handle;
  char*  name;

  fld = Id2Ptr(ID);
  handle = FldGetTextHandle(fld);
  FldSetTextHandle(fld, NULL);
  if (!handle) {
    handle = MemHandleNew(size*sizeof(char));
    name = MemHandleLock(handle);
    StrCopy(name, val);
    MemHandleUnlock(handle);
  } else {
    MemHandleResize(handle,size*sizeof(char));
    name = MemHandleLock(handle);
    StrCopy(name, val);
    MemHandleUnlock(handle);
  }
  FldSetTextHandle(fld, handle);
}

/**
 * routines handling the "Enter Team" form
 */
static void DrawFormEnterMann()
{
  Int16 nr = TeamsListGetSelection();

  if (nr == -1)
    return;

  MStatus = MemHandleLock(MStatusHandle);
  SetField(ID_FrmEnterMannFld,18,MStatus[nr].team);
  MemHandleUnlock(MStatusHandle);

  CtlSetValue(Id2Ptr(ID_FrmEnterMannButton+2), prefs.favTeam == nr+1);
}

static void SetEnteredMann(Int8 nr)
{
  Int16 size = 0, i;
  char *q, *s;
  UInt16 index = 1;

  if (prefs.options & readonly)
    return;

  MStatus = MemHandleLock(MStatusHandle);
  MStatus[nr].team = FldGetTextPtr(Id2Ptr(ID_FrmEnterMann));
  for (i=0; i<prefs.NumTeams; i++) {
    size += StrLen(MStatus[i].team)+1;
  }

  /*
   * resizing the record would overwrite MStatus, as we
   * actually use the pointers from the record. So we create
   * a new number "1" record and shift down all other records
   */
  q = MemHandleLock(DmNewRecord(db,&index,size));
  size = 0;
  for (i=0; i<prefs.NumTeams; i++) {
    s = MStatus[i].team;
    DmWrite(q,size,s,StrLen(s)+1);
    size += StrLen(s) + 1;
  }
  MemPtrUnlock(q);
  MemHandleUnlock(MStatusHandle);
  DmReleaseRecord(db,1,true);
  DmRemoveRecord(db,2);  // delete old number "1" record.
  ReadDatabase(db);
}

Boolean EnterMannFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Boolean update = false;
  Int8 nr;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    DrawFormEnterMann();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmEnterMannButton:
      nr = TeamsListGetSelection();
      SetEnteredMann(nr);
      update = true;
      if (CtlGetValue(Id2Ptr(ID_FrmEnterMannButton+2))) {
	prefs.favTeam = nr+1;
      }

    /* Cancel */
    case ID_FrmEnterMannButton+1:
      FrmReturnToForm(0);
      if (update) { 
	InitFormTeams();
	update = false;
      }
      RedrawList(ID_FrmTeams, ID_FrmTeamsList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling creation of a new database
 */
static void InitFrmNewDB()
{
  Int16 i, s[3] = { 19, 3, 3 };
  for (i=0; i<2; i++) {
    SetField(ID_FrmNewDBFld, s[i], "");
  }
}

/*
 * create database according to entries in form
 */
static void ReadFormNewDBValues()
{
  char *name, *nr, *rds;
  DmOpenRef db;
  BlDatabasePreferencesType newPrefs;
  Int16 n = 0, m = 0;

  nr = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+1));
  rds = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+2));

  if (nr)  n = StrAToI(nr);
  if (rds) m = StrAToI(rds);

  if (n < 2 || n > 24 || m < 1 || m > 90) {
    DisplayErrorMessage(ID_ErrorMessages+4);
    return;
  }

  name = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld));

  newPrefs.NumTeams     = n;
  newPrefs.NumMatchDays = m;
  newPrefs.matchDay     = 1;
  newPrefs.Version      = 6;
  newPrefs.options      = 0;
  newPrefs.favTeam      = 0;
  
  if ( name && (db = CreateDatabase(name)) ) {
    InitNewDatabase(db, &newPrefs);
    DmCloseDatabase(db);
  }
}

Boolean NewDBFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmNewDB();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    /* Ready */
    case ID_FrmNewDBButton:
      ReadFormNewDBValues();
      ChooseDB();

    /* Cancel */
    case ID_FrmNewDBButton+1:
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling renaming a database name
 */
static void InitFrmRenameLeague()
{
  UInt16 i = LstGetSelection(FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList));
  SetField(ID_FrmRenameLeagueFld, 26, Items[i]);
}

static void RenameDatabaseFile()
{
  char* name = FldGetTextPtr(Id2Ptr(ID_FrmRenameLeagueFld));
  if (name) {
    RenameDatabase(LstGetSelection(FrmGetListPtr(ID_FrmLeague, 
						 ID_FrmLeagueList)), name);
  }
}

Boolean RenameLeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmRenameLeague();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmRenameLeagueButton:
      RenameDatabaseFile();

    /* Cancel */
    case ID_FrmRenameLeagueButton+1:
      FrmReturnToForm(0);
      RedrawList(ID_FrmLeague, ID_FrmLeagueList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routine that handles the settings dialog
 */
Boolean SettingsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  ListPtr pLst = Id2Ptr(ID_FrmSettingsPopup);
  ControlPtr pCtl = Id2Ptr(ID_FrmSettingsButton+4);
  static Int8 options;
  static Int8 leagueOptions;

  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    /* read preferences and set form according to these */
    frm = FrmGetActiveForm();
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 2), 
		       appPrefs.options & editMode);
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 3), 
		       appPrefs.options & dontSaveEM);
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 5), 
		       prefs.options & readonly);
    CtlSetLabel(pCtl, LstGetSelectionText(pLst, prefs.options & twoPoints));
    options = appPrefs.options;
    leagueOptions = prefs.options;
    FrmDrawForm(frm);
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmSettingsButton: /* Ready => save settings */
      appPrefs.options = options; 
      prefs.options = leagueOptions;

    case ID_FrmSettingsButton+1: /* Cancel */
      FrmReturnToForm(0);
      handled = true; 
      break;

    case ID_FrmSettingsButton+2: /* Checkbox "Edit Mode" */
      handled = true; 
      options ^= editMode;
      break;

    case ID_FrmSettingsButton+3: /* Checkbox "Save Edit Mode" */
      handled = true; 
      options ^= dontSaveEM;
      break;

    case ID_FrmSettingsButton+4: /* Listbox 2/3 point rule */
      LstSetSelection(pLst, leagueOptions & twoPoints);
      LstPopupList(pLst);
      switch (LstGetSelection(pLst)) {
      case 0:
	leagueOptions &= ~twoPoints;
	handled = true; 
	break;
      case 1:
	leagueOptions |= twoPoints;
	handled = true; 
	break;
      }
      CtlSetLabel(pCtl,LstGetSelectionText(pLst,leagueOptions & twoPoints));
      break;
      
    case ID_FrmSettingsButton+5: /* read-only option */
      leagueOptions ^= readonly;
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines for handling League form;
 * update DBNr if entry in list was moved.
 */
static void updateDbNr(Int16 i, Int16 j)
{
  if (DBNr == i) { 
    DBNr = j;
  } else if (DBNr == j) {
    DBNr = i;
  }
}

Boolean LeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;
  ListPtr p;
  Int16 i;

  handled = false;

  switch (e->eType) {

  case frmOpenEvent:
    p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
    LstSetListChoices(p, Items, nrOfItems);
    LstSetHeight(p, nrOfItems>11?11:nrOfItems);
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmLeagueButton: /* Ready */
      handled = true; 
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawMatches(true);
      break;

    case ID_FrmLeagueButton+1: /* move entry up */
      p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
      i = LstGetSelection(p);
      if (i>0) {
	swapDbInfo(i,i-1);
	updateDbNr(i,i-1);
	LstSetListChoices(p, Items, nrOfItems);
	LstDrawList(p);
	LstSetSelection(p,i-1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+2: /* move entry down */
      p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
      i = LstGetSelection(p);
      if (i<nrOfItems-1) {
	swapDbInfo(i,i+1);
	updateDbNr(i,i+1);
	LstSetListChoices(p, Items, nrOfItems);
	LstDrawList(p);
	LstSetSelection(p,i+1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+3: /* Rename database name */
      FrmPopupForm(ID_FrmRenameLeague);
      handled = true;
    }

  default:
    break;
  }

  return handled;
}

Boolean HandicapEditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static UInt16 row, column;
  static ListPtr p;
  Int16 val, i, j;
  UInt16 sel;
  char buf[4], *cval;

  handled = false;
  
  switch (e->eType) {
    
  case frmOpenEvent:
    TblGetSelection(handicapTable, &row, &column);

    // match day
    val = TblGetItemInt(handicapTable, row, 1);
    StrIToA(buf, val+1);
    SetField(ID_FrmHandicapEditFld,4,buf);

    // value
    val = TblGetItemInt(handicapTable, row, 2);
    StrIToA(buf, val);
    /* check wether point or goal handicap and set the corresponding field */
    SetField(ID_FrmHandicapEditFld+TblGetItemInt(handicapTable,row,3)+1,
	     4,buf);

    val = TblGetItemInt(handicapTable, row, 0);
    if (val>0) {
      MemHandleLock(MStatusHandle);
      FrmCopyLabel(FrmGetActiveForm(), 
		   ID_FrmHandicapEditPopup, 
		   MStatus[val-1].team);
      MemHandleUnlock(MStatusHandle);
    }

    FrmDrawForm(FrmGetActiveForm());

    p = Id2Ptr(ID_FrmHandicapEditPopup+1);
    LstSetListChoices(p, TeamsList, prefs.NumTeams);
    LstSetHeight(p, prefs.NumTeams>10?10:prefs.NumTeams);
    if (val>0)
      LstSetSelection(p, val-1);

    handled = true; 
    break;
    
  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmHandicapEditButton: /* Ready */
      /*
       * read in fields and save changes into table
       */
      sel = LstGetSelection(p);
      if (sel != noListSelection) {
	TblSetItemInt(handicapTable, row, 0, sel+1);
      }
      cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld));
      if (cval && StrLen(cval)) {
	val = StrAToI(cval);
	if (val>0 && val<=prefs.NumMatchDays) {
	  TblSetItemInt(handicapTable, row, 1, val-1);
	}
      }
      cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld+1));
      if (cval && StrLen(cval)) {
	val = StrAToI(cval);
	if (val!=0) {
	  TblSetItemInt(handicapTable, row, 2, val);
	  TblSetItemInt(handicapTable, row, 3, 0);
	}
      } else {
	cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld+2));
	if (cval && StrLen(cval)) {
	  val = StrAToI(cval);
	  if (val!=0) {
	    TblSetItemInt(handicapTable, row, 2, val);
	    TblSetItemInt(handicapTable, row, 3, 1);
	  }
	}
      }
      goto FrmHandicapLabel;

    case ID_FrmHandicapEditButton+2: /* Delete */
      /*
       * delete handicap entry from table
       */
      handicapTableRows--;
      for (i=row; i<handicapTableRows; i++) {
	for (j=0; j<4; j++) {
	  TblSetItemInt(handicapTable, i, j, 
			TblGetItemInt(handicapTable, i+1, j));
	}
      }
      TblSetRowUsable(handicapTable,handicapTableRows,false);
      
    case ID_FrmHandicapEditButton+1: /* Cancel */
    FrmHandicapLabel:
      handled = true; 
      FrmReturnToForm(0);
      TblDrawTable(handicapTable);
      break;
      
    }
    break;

  default:
    break;
  }
  
  return handled;
}

static void DrawHandicapItem(void *tableP, Int16 row, Int16 column, 
			     RectangleType *bounds)
{
  char buf[25], *r;
  Int16 n = TblGetItemInt(tableP, row, column);

  FntSetFont(0);

  switch (column) {
  case 0:
    if (n>0) {
      MStatus = MemHandleLock(MStatusHandle);
      StrCopy(buf, MStatus[n-1].team);
      MemHandleUnlock(MStatusHandle);
    } else 
      StrCopy(buf,"-----");
    break;
  case 1:
    n+=1;
  case 2:
    StrIToA(buf,n);
    break;
  case 3:
    StrCopy(buf,r = GetString(ID_tSTR_HandicapType+n));
    MemPtrUnlock(r);
    break;
  }
  WinEraseRectangle(bounds, 0);
  WinDrawChars(buf, StrLen(buf), bounds->topLeft.x, bounds->topLeft.y);
}

/*
 * initialize table variables to default values;
 */
static void InitFormHandicap()
{
  Int16 i, j;

  handicapTable = Id2Ptr(ID_FrmHandicapTable);
  handicapTableRows = 0;

  for (i=0; i<TblGetNumberOfRows(handicapTable); i++) {
    TblSetRowSelectable(handicapTable, i, true);
    TblSetRowUsable(handicapTable, i, false);
    for (j=0; j<4; j++) {
      TblSetItemStyle(handicapTable, i, j, customTableItem);
      TblSetColumnUsable(handicapTable, j, true);
      TblSetCustomDrawProcedure(handicapTable, j, DrawHandicapItem);
    }
  }

  SetTeamsList();
}

/*
 * Callback function, that is called from LoopRecords().
 * Set TableItems according to handicap entry in database.
 */
static Boolean GetHandicap(handicapType *h, Int8 i)
{
  if (handicapTableRows<TblGetNumberOfRows(handicapTable)) {
    if (handicapTableRows<10)
      TblSetRowUsable(handicapTable, handicapTableRows, true);
    TblSetItemInt(handicapTable, handicapTableRows, 0, h->m);
    TblSetItemInt(handicapTable, handicapTableRows, 1, i);
    TblSetItemInt(handicapTable, handicapTableRows, 2, h->val);
    TblSetItemInt(handicapTable, handicapTableRows, 3, h->type);
    handicapTableRows++;
  }
  return true;
}

/*
 * Used for saving the handicaps; returns number of
 * entries for given match day.
 */
static Int8 numberOfHandicapEntries(Int8 spTag)
{
  Int16 i, nr = 0;
  for (i=0; i<handicapTableRows; i++) {
    if ( TblGetItemInt(handicapTable, i, 1) == spTag )
      nr++;
  }
  return nr;
}

/*
 * Used for saving the handicaps; fills in given structure
 * h with values from next entry for given match day.
 * Removes used line from Table.
 */
static void GetNextHandicap(handicapType *h, Int8 spTag)
{
  Int8 i, j, k;

  for (k=0; k<handicapTableRows; k++) {
    if ( TblGetItemInt(handicapTable, k, 1) == spTag ) {
      h->m    = TblGetItemInt(handicapTable, k, 0);
      h->val  = TblGetItemInt(handicapTable, k, 2);
      h->type = TblGetItemInt(handicapTable, k, 3);
      break;
    }
  }

  handicapTableRows--;
  for (i=k; i<handicapTableRows; i++) {
    for (j=0; j<4; j++) {
      TblSetItemInt(handicapTable, i, j, 
		    TblGetItemInt(handicapTable, i+1, j));
    }
  }
  
}

/*
 * Loops over all records and resizes them according number of
 * handicaps for each match day.
 */
static void SaveHandicaps()
{
  Int8 i, j;
  Int8 numH, numM;
  char *p;
  handicapType h;
  UInt32 size, offset;

  if (prefs.options & readonly)
    return;

  for (i=0; i<prefs.NumMatchDays; i++) {
    p = MemHandleLock(DmQueryRecord(db,i+2));
    numM = (Int8)*p;
    MemPtrUnlock(p);
    numH = numberOfHandicapEntries(i);

    size = 2 + numM*sizeof(matchType) + numH*sizeof(handicapType);
    p = MemHandleLock(DmResizeRecord(db,i+2,size));

    DmWrite(p,0,&numM,1);
    DmWrite(p,1,&numH,1);

    offset = 2 + numM*sizeof(matchType);
    
    for (j=0; j<numH; j++) {
      GetNextHandicap(&h, i);
      DmWrite(p,offset,&h,sizeof(handicapType));
      offset += sizeof(handicapType);
    }

    MemPtrUnlock(p);
    DmReleaseRecord(db, i+2, true);
  }

}

Boolean HandicapFormHandleEvent(EventPtr e)
{
  Boolean handled;
  handicapType h;
  static char from;

  handled = false;
  
  switch (e->eType) {
    
  case frmOpenEvent:
    /*
     * loop over all records, read in handicaps and show them
     * in a table.
     */
    InitFormHandicap();
    LoopRecords(NULL,GetHandicap);
    FrmDrawForm(FrmGetActiveForm());
    from = 0;
    handled = true; 
    break;
    
  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmHandicapButton: /* Ready */
      SaveHandicaps();
      handled = true; 
      FrmReturnToForm(0);
      break;
    
    case ID_FrmHandicapButton+1: /* New */
      /*
       * add a new entry to the table.
       */
      h.m = h.val = h.type = 0;
      GetHandicap(&h,0);
      TblDrawTable(handicapTable);
      handled = true; 
      break;

    case ID_FrmHandicapButton+2: /* Up */
      if (from > 0) {
	from -= 10;
	if (from < 0) 
	  from = 0;
        RedrawTable(handicapTable, from, handicapTableRows,
		    ID_FrmHandicapButton+2,
		    ID_FrmHandicapButton+3);
      }
      handled = true; 
      break;
    
    case ID_FrmHandicapButton+3: /* Down */
      if (from+10 < handicapTableRows) {
	from += 10;
	if (from+10>handicapTableRows) 
	  from = handicapTableRows - 10;
        RedrawTable(handicapTable, from, handicapTableRows,
		    ID_FrmHandicapButton+2,
		    ID_FrmHandicapButton+3);
      }
      handled = true; 
      break;
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (from > 0) {
	from -= 10;
	if (from < 0) 
	  from = 0;
        RedrawTable(handicapTable, from, handicapTableRows,
		    ID_FrmHandicapButton+2,
		    ID_FrmHandicapButton+3);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (from+10 < handicapTableRows) {
	from += 10;
	if (from+10>handicapTableRows) 
	  from = handicapTableRows - 10;
        RedrawTable(handicapTable, from, handicapTableRows,
		    ID_FrmHandicapButton+2,
		    ID_FrmHandicapButton+3);
      }
      handled = true; 
      break;
    }
    break;

  case tblSelectEvent:
    FrmPopupForm(ID_FrmHandicapEdit);
    handled = true; 
    break;

  default:
    break;
  }
  
  return handled;
}
