/*
 * Bundesliga is Copyright (c) 1997-2001 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

/**
 * Main Eventhandler Routinen
 */

UInt32 romversion;

static Boolean ApplicationHandleEvent(EventPtr e)
{
  Boolean handled;
  
  handled = false;

  if (e->eType==frmLoadEvent) {
    UInt16 frmId = e->data.frmLoad.formID;
    FormPtr frm = FrmInitForm(frmId);

    FrmSetActiveForm(frm);

    switch (frmId) {

    case ID_FrmBl:
      FrmSetEventHandler(frm, SpieltagFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmTable:
      FrmSetEventHandler(frm, TableFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmGoto:
      FrmSetEventHandler(frm, GotoFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmAbout:
      FrmSetEventHandler(frm, AboutFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmStatistics:
      FrmSetEventHandler(frm, StatisticsFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmEdit:
      FrmSetEventHandler(frm, EditFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmTeams:
      FrmSetEventHandler(frm, TeamsFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmSettings:
      FrmSetEventHandler(frm, SettingsFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmEnterMann:
      FrmSetEventHandler(frm, EnterMannFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmRenameLeague:
      FrmSetEventHandler(frm, RenameLeagueFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmNewDB:
      FrmSetEventHandler(frm, NewDBFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmHandicap:
      FrmSetEventHandler(frm, HandicapFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmHandicapEdit:
      FrmSetEventHandler(frm, HandicapEditFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmLeague:
      FrmSetEventHandler(frm, LeagueFormHandleEvent);
      handled = true; 
      break;

    case ID_FrmShowFixture:
      FrmSetEventHandler(frm, ShowFixtureFormHandleEvent);
      handled = true; 
      break;
    }
  }

  return handled;
}

static Int16 StartApplication(void)
{
  UInt16 size;
  Err err;
  
  size = sizeof(appPrefs);
  if ( !PrefGetAppPreferences('tpBL',0,&appPrefs,&size,false) ) {
    appPrefs.dbID     = 0;
    appPrefs.cardNo   = 0;
    appPrefs.Version  = 1;
    appPrefs.options  = 0;
  }
  
  if ( (err = OpenDatabase(appPrefs.cardNo, appPrefs.dbID)) ) {
    if ( err == (appErrorClass|tooOld) )
      DisplayErrorMessage(ID_ErrorMessages+1);
    if ( err == (appErrorClass|noDatabaseInstalled) )
      DisplayErrorMessage(ID_ErrorMessages+5);
    return err;
  }

  matchDay = prefs.matchDay;
  return 0;
}

static void EventLoop(void)
{
  EventType e;
  short err;

  do {
    EvtGetEvent(&e, evtWaitForever);
    if (! SysHandleEvent (&e))
      if (! MenuHandleEvent (NULL, &e, &err))
        if (! ApplicationHandleEvent (&e))
          FrmDispatchEvent (&e);
  } while (e.eType != appStopEvent);
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
  Err err;
  
  if (cmd == sysAppLaunchCmdNormalLaunch) {

    err = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
    if (FeatureSet20) {

      if (StartApplication())
	return 0;

      FrmGotoForm(ID_FrmBl);
      
      EventLoop();
      
      FrmCloseAllForms();
      SaveStatus(DBNr);

      if (MStatus)
	MemHandleFree(MStatusHandle);

    } else {

      DisplayInformationMessage(ID_InformationMessages);
      AppLaunchWithCommand(sysFileCDefaultApp,
			   sysAppLaunchCmdNormalLaunch, NULL);
    }

  }

  return 0;
}
